@@ -19,9 +19,9 @@ import xarray
 from datacube.utils import geometry
 from datacube.utils.masking import mask_to_dict
 from pandas import Timestamp
+from rasterio.features import rasterize
 from rasterio.io import MemoryFile
 from rasterio.warp import Resampling
-from skimage.draw import polygon as skimg_polygon
 
 from datacube_ows.cube_pool import cube
 from datacube_ows.mv_index import MVSelectOpts, mv_search
@@ -31,6 +31,8 @@ from datacube_ows.ogc_utils import (ConfigException, dataset_center_time,
                                     xarray_image_as_png)
 from datacube_ows.ows_configuration import get_config
 from datacube_ows.query_profiler import QueryProfiler
+from datacube_ows.resource_limits import ResourceLimited
+from datacube_ows.startup_utils import CredentialManager
 from datacube_ows.utils import log_call
 from datacube_ows.wms_utils import (GetFeatureInfoParameters, GetMapParameters,
                                     img_coords_to_geopoint, solar_correct_data)
@@ -54,7 +56,7 @@ class ProductBandQuery:
     def __str__(self):
         return f"Query bands {self.bands} from products {self.products}"
 
-    def  __hash__(self):
+    def __hash__(self):
         return hash(self.key)
 
     @classmethod
@@ -131,7 +133,7 @@ class DataStacker:
         if style:
             self._needed_bands = list(style.needed_bands)
         elif bands:
-            self._needed_bands = [self._product.band_idx.band(b) for b in bands]
+            self._needed_bands = [self._product.band_idx.locale_band(b) for b in bands]
         else:
             self._needed_bands = list(self._product.band_idx.native_bands.index)
 
@@ -179,7 +181,7 @@ class DataStacker:
             queries = ProductBandQuery.full_layer_queries(self._product, self.needed_bands())
         else:
             # Just take needed bands.
-            queries = ProductBandQuery.full_layer_queries(self._product, self.needed_bands())
+            queries = [ProductBandQuery.simple_layer_query(self._product, self.needed_bands())]
 
         if point:
             geom = point
@@ -322,12 +324,16 @@ class DataStacker:
     # Read data for given datasets and measurements per the output_geobox
     @log_call
     def read_data(self, datasets, measurements, geobox, resampling=Resampling.nearest, fuse_func=None):
-        return datacube.Datacube.load_data(
-                datasets,
-                geobox,
-                measurements=measurements,
-                fuse_func=fuse_func)
-
+        CredentialManager.check_cred()
+        try:
+            return datacube.Datacube.load_data(
+                    datasets,
+                    geobox,
+                    measurements=measurements,
+                    fuse_func=fuse_func)
+        except Exception as e:
+            _LOG.error("Error (%s) in load_data: %s", e.__class__.__name__, str(e))
+            raise
     # Read data for single datasets and measurements per the output_geobox
     @log_call
     def read_data_for_single_dataset(self, dataset, measurements, geobox, resampling=Resampling.nearest, fuse_func=None):
@@ -336,11 +342,16 @@ class DataStacker:
             dc_datasets = datacube.Datacube.group_datasets(datasets, 'solar_day')
         else:
             dc_datasets = datacube.Datacube.group_datasets(datasets, 'time')
-        return datacube.Datacube.load_data(
-            dc_datasets,
-            geobox,
-            measurements=measurements,
-            fuse_func=fuse_func)
+        CredentialManager.check_cred()
+        try:
+            return datacube.Datacube.load_data(
+                dc_datasets,
+                geobox,
+                measurements=measurements,
+                fuse_func=fuse_func)
+        except Exception as e:
+            _LOG.error("Error (%s) in load_data: %s", e.__class__.__name__, str(e))
+            raise
 
 
 def datasets_in_xarray(xa):
@@ -372,6 +383,8 @@ def user_date_sorter(odc_dates, geom, user_dates):
     )
     return xrresult
 
+class EmptyResponse(Exception):
+    pass
 
 @log_call
 def get_map(args):
@@ -389,107 +402,109 @@ def get_map(args):
                                WMSException.INVALID_DIMENSION_VALUE, locator="Time parameter")
     qprof["n_dates"] = n_dates
     with cube() as dc:
-        if not dc:
-            raise WMSException("Database connectivity failure")
-        # Tiling.
-        stacker = DataStacker(params.product, params.geobox, params.times, params.resampling, style=params.style)
-        zoomed_out = params.zf < params.product.min_zoom
-        qprof["zoom_factor"] = params.zf
-        qprof.start_event("count-datasets")
-        n_datasets = stacker.datasets(dc.index, mode=MVSelectOpts.COUNT)
-        qprof.end_event("count-datasets")
-        qprof["n_datasets"] = n_datasets
-        too_many_datasets = (params.product.max_datasets_wms > 0
-                             and n_datasets > params.product.max_datasets_wms
-                             )
-        if qprof.active:
-            qprof["datasets"] = stacker.datasets(dc.index, mode=MVSelectOpts.IDS)
-        if too_many_datasets or zoomed_out:
-            stacker.resource_limited = True
-            qprof["too_many_datasets"] = too_many_datasets
-            qprof["zoomed_out"] = zoomed_out
-
-        if stacker.resource_limited and not params.product.low_res_product_names:
-            qprof.start_event("extent-in-query")
-            extent = stacker.datasets(dc.index, mode=MVSelectOpts.EXTENT)
-            qprof.end_event("extent-in-query")
-            if extent is None:
-                qprof["write_action"] = "No extent: Write Empty"
-                qprof.start_event("write")
-                body = _write_empty(params.geobox)
-                qprof.end_event("write")
+        try:
+            if not dc:
+                raise WMSException("Database connectivity failure")
+            # Tiling.
+            stacker = DataStacker(params.product, params.geobox, params.times, params.resampling, style=params.style)
+            qprof["zoom_factor"] = params.zf
+            qprof.start_event("count-datasets")
+            n_datasets = stacker.datasets(dc.index, mode=MVSelectOpts.COUNT)
+            qprof.end_event("count-datasets")
+            qprof["n_datasets"] = n_datasets
+            qprof["zoom_level_base"] = params.resources.base_zoom_level
+            qprof["zoom_level_adjusted"] = params.resources.load_adjusted_zoom_level
+            try:
+                params.product.resource_limits.check_wms(n_datasets, params.zf, params.resources)
+            except ResourceLimited as e:
+                stacker.resource_limited = True
+                qprof["resource_limited"] = str(e)
+            if qprof.active:
+                qprof["datasets"] = stacker.datasets(dc.index, mode=MVSelectOpts.IDS)
+            if stacker.resource_limited and not params.product.low_res_product_names:
+                qprof.start_event("extent-in-query")
+                extent = stacker.datasets(dc.index, mode=MVSelectOpts.EXTENT)
+                qprof.end_event("extent-in-query")
+                if extent is None:
+                    qprof["write_action"] = "No extent: Write Empty"
+                    raise EmptyResponse()
+                else:
+                    qprof["write_action"] = "Polygon"
+                    qprof.start_event("write")
+                    body = _write_polygon(
+                        params.geobox,
+                        extent,
+                        params.product.resource_limits.zoom_fill,
+                        params.product)
+                    qprof.end_event("write")
+            elif n_datasets == 0:
+                qprof["write_action"] = "No datasets: Write Empty"
+                raise EmptyResponse()
             else:
-                qprof["write_action"] = "Polygon"
-                qprof.start_event("write")
-                body = _write_polygon(
-                    params.geobox,
-                    extent,
-                    params.product.zoom_fill,
-                    params.product)
-                qprof.end_event("write")
-        elif n_datasets == 0:
-            qprof["write_action"] = "No datsets: Write Empty"
+                if stacker.resource_limited:
+                    qprof.start_event("count-summary-datasets")
+                    qprof["n_summary_datasets"] = stacker.datasets(dc.index, mode=MVSelectOpts.COUNT)
+                    qprof.end_event("count-summary-datasets")
+                qprof.start_event("fetch-datasets")
+                datasets = stacker.datasets(dc.index)
+                for flagband, dss in datasets.items():
+                    if not dss.any():
+                        _LOG.warning("Flag band %s returned no data", str(flagband))
+                    if len(dss.time) != n_dates and flagband.main:
+                        qprof["write_action"] = f"{n_dates} requested, only {len(dss.time)} found - returning empty image"
+                        raise EmptyResponse()
+                qprof.end_event("fetch-datasets")
+                _LOG.debug("load start %s %s", datetime.now().time(), args["requestid"])
+                qprof.start_event("load-data")
+                data = stacker.data(datasets)
+                qprof.end_event("load-data")
+                _LOG.debug("load stop %s %s", datetime.now().time(), args["requestid"])
+                qprof.start_event("build-masks")
+                td_masks = []
+                for npdt in data.time.values:
+                    td = data.sel(time=npdt)
+                    td_ext_mask = None
+                    for band in params.style.needed_bands:
+                        if band not in params.style.flag_bands:
+                            if params.product.data_manual_merge:
+                                if td_ext_mask is None:
+                                    td_ext_mask = ~numpy.isnan(td[band])
+                                else:
+                                    td_ext_mask &= ~numpy.isnan(td[band])
+                            else:
+                                for f in params.product.extent_mask_func:
+                                    if td_ext_mask is None:
+                                        td_ext_mask = f(td, band)
+                                    else:
+                                        td_ext_mask &= f(td, band)
+                    if params.product.data_manual_merge:
+                        td_ext_mask = xarray.DataArray(td_ext_mask)
+                    td_masks.append(td_ext_mask)
+                extent_mask = xarray.concat(td_masks, dim=data.time)
+                qprof.end_event("build-masks")
+
+                if not data:
+                    qprof["write_action"] = "No Data: Write Empty"
+                    raise EmptyResponse()
+                else:
+                    qprof["write_action"] = "Write Data"
+                    if mdh and  mdh.preserve_user_date_order:
+                        sorter = user_date_sorter(data.time.values,
+                                              params.geobox.geographic_extent,
+                                              params.times)
+                        data = data.sortby(sorter)
+                        extent_mask = extent_mask.sortby(sorter)
+
+                    body = _write_png(data, params.style, extent_mask, qprof)
+        except EmptyResponse:
             qprof.start_event("write")
             body = _write_empty(params.geobox)
             qprof.end_event("write")
-        else:
-            if stacker.resource_limited:
-                qprof.start_event("count-summary-datasets")
-                qprof["n_summary_datasets"] = stacker.datasets(dc.index, mode=MVSelectOpts.COUNT)
-                qprof.end_event("count-summary-datasets")
-            qprof.start_event("fetch-datasets")
-            datasets = stacker.datasets(dc.index)
-            for flagband, dss in datasets.items():
-                if not dss.any():
-                    _LOG.warning("Flag band %s returned no data", str(flagband))
-            qprof.end_event("fetch-datasets")
-            _LOG.debug("load start %s %s", datetime.now().time(), args["requestid"])
-            qprof.start_event("load-data")
-            data = stacker.data(datasets)
-            qprof.end_event("load-data")
-            _LOG.debug("load stop %s %s", datetime.now().time(), args["requestid"])
-            qprof.start_event("build-masks")
-            td_masks = []
-            for npdt in data.time.values:
-                td = data.sel(time=npdt)
-                td_ext_mask = None
-                for band in params.style.needed_bands:
-                    if band not in params.style.flag_bands:
-                        if params.product.data_manual_merge:
-                            if td_ext_mask is None:
-                                td_ext_mask = ~numpy.isnan(td[band])
-                            else:
-                                td_ext_mask &= ~numpy.isnan(td[band])
-                        else:
-                            for f in params.product.extent_mask_func:
-                                if td_ext_mask is None:
-                                    td_ext_mask = f(td, band)
-                                else:
-                                    td_ext_mask &= f(td, band)
-                if params.product.data_manual_merge:
-                    td_ext_mask = xarray.DataArray(td_ext_mask)
-                td_masks.append(td_ext_mask)
-            extent_mask = xarray.concat(td_masks, dim=data.time)
-            qprof.end_event("build-masks")
-
-            if not data:
-                qprof["write_action"] = "No Data: Write Empty"
-                body = _write_empty(params.geobox)
-            else:
-                qprof["write_action"] = "Write Data"
-                if mdh and  mdh.preserve_user_date_order:
-                    sorter = user_date_sorter(data.time.values,
-                                          params.geobox.geographic_extent,
-                                          params.times)
-                    data = data.sortby(sorter)
-                    extent_mask = extent_mask.sortby(sorter)
-
-                body = _write_png(data, params.style, extent_mask, params.geobox, qprof)
 
     if params.ows_stats:
         return json_response(qprof.profile())
     else:
-        return png_response(body, extra_headers=params.product.wms_cache_rules.cache_headers(n_datasets))
+        return png_response(body, extra_headers=params.product.resource_limits.wms_cache_rules.cache_headers(n_datasets))
 
 
 def png_response(body, cfg=None, extra_headers=None):
@@ -504,7 +519,7 @@ def png_response(body, cfg=None, extra_headers=None):
 
 
 @log_call
-def _write_png(data, style, extent_mask, geobox, qprof):
+def _write_png(data, style, extent_mask, qprof):
     qprof.start_event("combine-masks")
     mask = style.to_mask(data, extent_mask)
     qprof.end_event("combine-masks")
@@ -512,7 +527,13 @@ def _write_png(data, style, extent_mask, geobox, qprof):
     img_data = style.transform_data(data, mask)
     qprof.end_event("apply-style")
     qprof.start_event("write")
-    image = xarray_image_as_png(img_data, mask)
+    # If time dimension is present animate over it.
+    # Verified using : https://docs.dea.ga.gov.au/notebooks/Frequently_used_code/Animated_timeseries.html
+    mdh = style.get_multi_date_handler(img_data)
+    if mdh:
+        image = xarray_image_as_png(img_data, loop_over='time', animate=True, frame_duration=mdh.frame_duration)
+    else:
+        image = xarray_image_as_png(img_data)
     qprof.end_event("write")
     return image
 
@@ -560,18 +581,17 @@ def _write_polygon(geobox, polygon, zoom_fill, layer):
         data = numpy.full([geobox.height, geobox.width], fill_value=1, dtype="uint8")
     else:
         data = numpy.zeros([geobox.height, geobox.width], dtype="uint8")
-        coordinates_list = get_coordlist(polygon, layer.name)
-        for polygon_coords in coordinates_list:
-            pixel_coords = [~geobox.transform * coords for coords in polygon_coords[0]]
-            rs, cs = skimg_polygon([c[1] for c in pixel_coords], [c[0] for c in pixel_coords],
-                                   shape=[geobox.width, geobox.height])
-            data[rs, cs] = 1
-
+        data = rasterize(shapes=[polygon],
+                          fill=0,
+                          default_value=2,
+                          out=data,
+                          transform=geobox.affine
+                        )
     with MemoryFile() as memfile:
         with memfile.open(driver='PNG',
                           width=geobox.width,
                           height=geobox.height,
-                          count=len(zoom_fill),
+                          count=4,
                           transform=None,
                           nodata=0,
                           dtype='uint8') as thing:
