@@ -8,91 +8,10 @@
 
 # Example configuration file for datacube_ows.
 #
-# For detailed formal documentation see:
+# For detailed and up to date formal documentation see:
 #
 #   https://datacube-ows.readthedocs.io/en/latest/configuration.html
-#
-# OVERVIEW
-#
-# This file forms the primary documentation for the configuration file format at this stage.
-#
-# The actual configuration is held in a single serialisable object that can be directly
-# declared as a python object or imported from JSON.
-#
-# WHERE IS CONFIGURATION READ FROM?
-#
-# Configuration is read by default from the ows_cfg object in datacube_ows/ows_cfg.py
-#
-# but this can be overridden by setting the $DATACUBE_OWS_CFG environment variable.
-#
-# $DATACUBE_OWS_CFG is interpreted as follows (first matching alternative applies):
-#
-# 1. Has a leading slash, e.g. "/opt/odc_ows_cfg/odc_ows_cfg_prod.json"
-#    Config loaded as json from absolute file path.
-#
-# 2. Contains a slash, e.g. "configs/prod.json"
-#    Config loaded as json from relative file path.
-#
-# 3. Begins with an open brace "{", e.g. "{...}"
-#    Config loaded directly from the environment variable as json (not recommended)
-#
-# 4. Ends in ".json", e.g. "cfg_prod.json"
-#    Config loaded as json from file in working directory.
-#
-# 5. Contains a dot (.), e.g. "package.sub_package.module.cfg_object_name"
-#    Imported as python object (expected to be a dictionary).
-#    N.B. It is up to you that the Python file in question is in your Python path.
-#
-# 6. Valid python object name, e.g. "cfg_prod"
-#    Imported as python object from named object in datacube_ows/ows_cfg.py
-#
-# 7. Blank or not set
-#    Default to import ows_cfg object in datacube_ows/ows_cfg.py as described above.
-#
-# REUSING CHUNKS OF CONFIGURATION
-#
-# Often it is desirable to re-use chunks of configuration in multiple places.  E.g. a set
-# of related data products may share a band index or style definition configurations.
-#
-# If you are loading config as a Python object, this is trivial, as demonstrated in this
-# example file.
-#
-# If you want to reuse chunks of config in json, or wish to combine bits of json config
-# with bits of python config, the following convention applies in both Python and JSON
-# configuration:
-#
-# Any JSON or Python element that forms the full configuration tree or a subset of it,
-# can be supplied in any of the following ways:
-#
-# 1. Directly embed the config content:
-#       {
-#           "a_cfg_entry": 1,
-#           "another_entry": "llama",
-#       }
-#
-# 2. Include a python object (by FQN):
-#       {
-#           "include": "path.to.module.object",
-#           "type": "python"
-#       }
-#
-#       N.B. It is up to you to make sure the included Python file is in your Python Path.
-#            Relative Python imports are not supported.
-#
-# 3. Include a JSON file (by absolute or relative file path):
-#       {
-#           "include": "path/to/file.json",
-#           "type": "json"
-#       }
-#
-#       N.B. Resolution of relative file paths is done in the following order:
-#           a) Relative to the working directory of the web app.
-#           b) If a JSON file is being included from another JSON file, relative to
-#              directory in which the including file resides.
-#
-# Note that this does not just apply to dictionaries. Either of the above include dictionaries
-# could expand to an array, or even to single integer or string.
-#
+
 # THIS EXAMPLE FILE
 #
 # In this example file, there are some reusable code chunks defined at the top.  The actual
@@ -816,6 +735,247 @@ style_ndvi_delta = {
     ]
 }
 
+# Examples of non-linear colour-ramped style with multi-date animation support.
+style_ndvi_anim = {
+    "name": "ndvi_anim",
+    "title": "NDVI Animation",
+    "abstract": "Normalised Difference Vegetation Index - with animation support",
+    "index_function": {
+        "function": "datacube_ows.band_utils.norm_diff",
+        "mapped_bands": True,
+        "kwargs": {
+            "band1": "nir",
+            "band2": "red"
+        }
+    },
+    "needed_bands": ["red", "nir"],
+    # The color ramp for single-date requests - same as ndvi style example above
+    "color_ramp": [
+        {
+            "value": -0.0,
+            "color": "#8F3F20",
+            "alpha": 0.0
+        },
+        {
+            "value": 0.0,
+            "color": "#8F3F20",
+            "alpha": 1.0
+        },
+        {
+            "value": 0.1,
+            "color": "#A35F18"
+        },
+        {
+            "value": 0.2,
+            "color": "#B88512"
+        },
+        {
+            "value": 0.3,
+            "color": "#CEAC0E"
+        },
+        {
+            "value": 0.4,
+            "color": "#E5D609"
+        },
+        {
+            "value": 0.5,
+            "color": "#FFFF0C"
+        },
+        {
+            "value": 0.6,
+            "color": "#C3DE09"
+        },
+        {
+            "value": 0.7,
+            "color": "#88B808"
+        },
+        {
+            "value": 0.8,
+            "color": "#529400"
+        },
+        {
+            "value": 0.9,
+            "color": "#237100"
+        },
+        {
+            "value": 1.0,
+            "color": "#114D04"
+        }
+    ],
+    "include_in_feature_info": True,
+    "legend": {
+        # Show the legend (default True for colour ramp styles)
+        "show_legend": True,
+        # Example config for colour ramp style auto-legend generation.
+
+        # The range covered by the legend.
+        # Defaults to the first and last non transparent (alpha != 0.0)
+        # entry in the explicit colour ramp, or the values in the range parameter.
+        # It is recommended that values be supplied as integers or strings rather
+        # than floating point.
+        "begin": "0.0",
+        "end": "1.0",
+        # Ticks.
+        # One of the following alternatives.  All the examples below result in the same tick behaviour, given
+        # the begin and end values above.
+        #
+        # 1. Regularly spaced ticks, by size, starting from the begin tick.
+        "ticks_every": "0.2",
+        # 2. Regularly spaced ticks, by number of ticks, not counting the begin tick, but including the end tick. (int)
+        # "tick_count": 5,
+        # 3. Explicit ticks
+        # "ticks": [ "0.0", "0.2", "0.4", "0.6". "0.8", "1.0"]
+        # Default is a tick_count of 1, which means only the begin and end ticks.
+        # Legend title.  Defaults to the style name.
+        "title": "This is not a legend",
+
+        # Units
+        # added to title of legend in parenthesis, default is to not display units.  To emulate
+        # the previous default behaviour use:
+        "units": "unitless",
+
+        # decimal_places. 1 for "1.0" style labels, 2 for "1.00" and 0 for "1", etc.
+        # (default 1)
+        "decimal_places": 1,
+
+        # tick_labels
+        # Labels for individual ticks can be customised"
+        "tick_labels": {
+            # The special entry "default" allows setting
+            # a prefix and/or suffix for all labels.
+            # Default is no prefix or suffix
+            "default": {
+                # E.g. this encloses every tick label in parentheses.
+                "prefix": "(",
+                "suffix": ")",
+            },
+            # Other entries override the label for individual ticks.
+            # If they do not match a tick, as defined by the tick behaviour
+            # described above, the entry is ignored.  If you are having trouble
+            # getting the right tick value, use the "ticks" option to explicitly
+            # declare your tick locations and make sure you use strings instead of
+            # floats.
+            # The default prefix and suffix can be over-ridden.
+            "0.0": {
+                # E.g. to remove the parentheses for the 0.0 tick
+                "prefix": "",
+                "suffix": "",
+            },
+            # Or the label can changed.  Note that the default prefix and suffix
+            # are still applied unless explicitly over-ridden.
+            # E.g. To display "(max)" for the 1.0 tick:
+            "1.0": {
+                "label": "max"
+            }
+        },
+
+        # MatPlotLib rcparams options.
+        # Defaults to {} (i.e. matplotlib defaults)
+        # See https://matplotlib.org/3.2.2/tutorials/introductory/customizing.html
+        "rcParams": {
+                 "lines.linewidth": 2,
+                 "font.weight": "bold",
+        },
+
+        # Image size (in "inches").
+        # Matplotlib's default dpi is 100, so measured in hundreds of pixels unless the dpi
+        # is over-ridden by the rcParams above.
+        # Default is 4x1.25, i.e. 400x125 pixels
+        "width": 4,
+        "height": 1.25,
+
+        # strip_location
+        # The location and size of the coloured strip, in format:
+        #  [ left, bottom, width, height ], as passed to Matplotlib Figure.add_axes function.
+        # All values as fractions of the width and height.  (i.e. between 0.0 and 1.0)
+        # The default is:
+        "strip_location": [0.05, 0.5, 0.9, 0.15]
+    },
+    # Define behaviour(s) for multi-date requests. If not declared, style only supports single-date requests.
+    "multi_date": [
+        # A multi-date handler.  Different handlers can be declared for different numbers of dates in a request.
+        {
+            # The count range for which this handler is to be used - a tuple of two ints, the smallest and
+            # largest date counts for which this handler will be used.  Required.
+            # For animations consider the RAM usage on OWS pods and keep the maximum time steps reasonable
+            "allowed_count_range": [2, 10],
+            # Re-sort data returned from the ODC so the order of date coordinates in the "time" dimension
+            # matches the date order supplied by the user in the WMS request.
+            # Optional. Defaults to False (date coordinates in the "time" dimension are always sorted chronologically)
+            "preserve_user_date_order": True,
+            # Flag that an animation should be generated. Without this only the most recent frame is returned. Optional.
+            "animate": True,
+            # A function, expressed in the standard format as described elsewhere in this example file.
+            # The function is assumed to take one arguments, an xarray Dataset.
+            # The function returns an xarray Dataset with a with multiple-bands per time step, output is defined
+            # by colour ramp below
+            "aggregator_function": {
+                "function": "datacube_ows.band_utils.multi_date_pass"
+            },
+            # The color ramp for single-date requests - same as ndvi style example above
+            "color_ramp": [
+                {
+                    "value": -0.0,
+                    "color": "#8F3F20",
+                    "alpha": 0.0
+                },
+                {
+                    "value": 0.0,
+                    "color": "#8F3F20",
+                    "alpha": 1.0
+                },
+                {
+                    "value": 0.1,
+                    "color": "#A35F18"
+                },
+                {
+                    "value": 0.2,
+                    "color": "#B88512"
+                },
+                {
+                    "value": 0.3,
+                    "color": "#CEAC0E"
+                },
+                {
+                    "value": 0.4,
+                    "color": "#E5D609"
+                },
+                {
+                    "value": 0.5,
+                    "color": "#FFFF0C"
+                },
+                {
+                    "value": 0.6,
+                    "color": "#C3DE09"
+                },
+                {
+                    "value": 0.7,
+                    "color": "#88B808"
+                },
+                {
+                    "value": 0.8,
+                    "color": "#529400"
+                },
+                {
+                    "value": 0.9,
+                    "color": "#237100"
+                },
+                {
+                    "value": 1.0,
+                    "color": "#114D04"
+                }
+            ],
+            "legend": {
+                # Legend only covers positive part of ramp.
+                "begin": "0.0",
+                "end": "1.0"
+            },
+            # The feature info label for the multi-date animation performs a pixel drill.
+            "feature_info_label": "ndvi_anim"
+        }
+    ]
+}
+
 # Examples of Matplotlib Color-Ramp styles
 style_deform = {
     "name": "deform",
@@ -1546,6 +1706,18 @@ ows_cfg = {
                     # * "month" (for monthly summary datasets)
                     # * "year" (for annual summary datasets)
                     "time_resolution": "raw",
+                    # The "native" CRS.  (as used for resource management calculations and WCS metadata)
+                    # (Used for resource management calculations and WCS metadata)
+                    # Must be in the global "published_CRSs" list.
+                    # Can be omitted if the product has a single native CRS, as this will be used in preference.
+                    "native_crs": "EPSG:3577",
+                    # The native resolution (x,y)
+                    # (Used for resource management calculations and WCS metadata)
+                    # This is the number of CRS units (e.g. degrees, metres) per pixel in the horizontal
+                    # and vertical directions for the native CRS.
+                    # Can be omitted if the product has a single native resolution, as this will be used in preference.
+                    # E.g. for EPSG:3577; (25.0,25.0) for Landsat-8 and (10.0,10.0 for Sentinel-2)
+                    "native_resolution": [25.0, 25.0],
                     "flags": {
                         # Data may include flags that mark which pixels have missing or poor-quality data,
                         # or contain cloud, or cloud-shadow, etc.  This section describes how
@@ -1634,14 +1806,7 @@ ows_cfg = {
                     # If the WCS section is not supplied, then this named layer will NOT appear as a WCS
                     # coverage (but will still be a layer in WMS and WMTS).
                     "wcs": {
-                        # The "native" CRS for WCS. Must be in the global "published_CRSs" list.
-                        # Can be omitted if the product has a single native CRS, as this will be used in preference.
-                        "native_crs": "EPSG:3577",
-                        # The resolution (x,y) for WCS.  Required for WCS-enabled layers.
-                        # This is the number of CRS units (e.g. degrees, metres) per pixel in the horizontal
-                        # and vertical # directions for the native resolution.
-                        # E.g. for EPSG:3577; (25.0,25.0) for Landsat-8 and (10.0,10.0 for Sentinel-2)
-                        "native_resolution": [25.0, 25.0],
+
                         # The default bands for a WCS request.
                         # 1. Must be provided if WCS is activated.
                         # 2. Must contain at least one band.
@@ -1756,6 +1921,7 @@ ows_cfg = {
                     "product_name": "ls8_level1_usgs",
                     "bands": landsat8_bands,
                     "resource_limits": standard_resource_limits,
+                    # native_crs and native_resolution taken from product meta-data
                     "flags": {
                         "band": "quality",
                         "ignore_time": False,
@@ -1781,8 +1947,6 @@ ows_cfg = {
                         "apply_solar_corrections": True
                     },
                     "wcs": {
-                        "native_crs": "EPSG:3857",
-                        "native_resolution": [25.0, 25.0],
                         "default_bands": ["red", "green", "blue"],
                     },
                     "styling": {
@@ -1809,13 +1973,13 @@ ows_cfg = {
                     "bands": {"frequency": []},
                     "resource_limits": standard_resource_limits,
                     "flags": None,
+                    "native_crs": "EPSG:3857",
+                    "native_resolution": [25.0, 25.0],
                     "image_processing": {
                         "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
                         "fuse_func": "datacube_ows.wms_utils.wofls_fuser",
                     },
                     "wcs": {
-                        "native_crs": "EPSG:3857",
-                        "native_resolution": [25.0, 25.0],
                         "default_bands": ["frequency"],
                     },
                     "styling": {
@@ -1849,6 +2013,8 @@ ows_cfg = {
                     "resource_limits": standard_resource_limits,
                     # Near Real Time datasets are being regularly updated - do not cache ranges in memory.
                     "dynamic": True,
+                    "native_crs": "EPSG:3577",
+                    "native_resolution": [10.0, 10.0],
                     "flags": {
                         "band": "quality",
                         "ignore_time": False,
@@ -1863,8 +2029,6 @@ ows_cfg = {
                         "apply_solar_corrections": False,
                     },
                     "wcs": {
-                        "native_crs": "EPSG:3577",
-                        "native_resolution": [10.0, 10.0],
                         "default_bands": ["red", "green", "blue"],
                     },
                     "identifiers": {
@@ -1909,8 +2073,6 @@ ows_cfg = {
                 "apply_solar_corrections": False,
             },
             "wcs": {
-                "native_crs": "EPSG:3577",
-                "native_resolution": [25.0, 25.0],
                 "default_bands": ["canopy_cover_class"],
             },
             "identifiers": {
