@@ -36,6 +36,7 @@ from datacube_ows.ogc_utils import (ConfigException, FunctionWrapper,
                                     create_geobox, day_summary_date_range,
                                     local_solar_date_range, month_date_range,
                                     year_date_range)
+from datacube_ows.resource_limits import OWSResourceManagementRules
 from datacube_ows.styles import StyleDef
 from datacube_ows.tile_matrix_sets import TileMatrixSet
 from datacube_ows.utils import group_by_statistical
@@ -66,20 +67,30 @@ def read_config(path=None):
     return cfg_expand(cfg, cwd=cwd)
 
 
-class BandIndex(OWSConfigEntry):
+class BandIndex(OWSMetadataConfig):
+    METADATA_DEFAULT_BANDS = True
+    METADATA_TITLE = False
+    METADATA_ABSTRACT = False
+
     def __init__(self, layer, band_cfg):
+        if band_cfg is None:
+            band_cfg = {}
         super().__init__(band_cfg)
+        self.band_cfg = band_cfg
         self.product = layer
         self.product_name = layer.name
-        if band_cfg is None:
-            self.band_cfg = {}
-        else:
-            self.band_cfg = band_cfg
+        self.parse_metadata(band_cfg)
         self._idx = {}
         self.add_aliases(self.band_cfg)
         self.declare_unready("native_bands")
         self.declare_unready("_nodata_vals")
 
+    def global_config(self):
+        return self.product.global_config()
+
+    def get_obj_label(self):
+        return self.product.get_obj_label() + ".bands"
+
     def add_aliases(self, cfg):
         for b, aliases in cfg.items():
             if b in self._idx:
@@ -112,12 +123,19 @@ class BandIndex(OWSConfigEntry):
             return self._idx[name_alias]
         raise ConfigException(f"Unknown band name/alias: {name_alias} in layer {self.product.name}")
 
+    def locale_band(self, name_alias):
+        try:
+            return self.band(name_alias)
+        except ConfigException:
+            pass
+        for b in self.band_cfg.keys():
+            if name_alias == self.band_label(b):
+                return b
+        raise ConfigException(f"Unknown band: {name_alias} in layer {self.product.name}")
+
     def band_label(self, name_alias):
-        name = self.band(name_alias)
-        if self.band_cfg[name]:
-            return self.band_cfg[name][0]
-        else:
-            return name
+        canonical_name = self.band(name_alias)
+        return self.read_local_metadata(canonical_name)
 
     def nodata_val(self, name_alias):
         name = self.band(name_alias)
@@ -176,7 +194,6 @@ class SuppURL(OWSConfigEntry):
         self.format = cfg["format"]
 
 
-
 class OWSLayer(OWSMetadataConfig):
     METADATA_KEYWORDS = True
     METADATA_ATTRIBUTION = True
@@ -237,12 +254,15 @@ class OWSFolder(OWSLayer):
             raise ConfigException("No layers section in folder layer %s" % self.title)
         child = 0
         for lyr_cfg in cfg["layers"]:
-            try:
-                lyr = parse_ows_layer(lyr_cfg, global_cfg=global_cfg, parent_layer=self, sibling=child)
-                self.unready_layers.append(lyr)
-            except ConfigException as e:
-                _LOG.error("Could not parse layer: %s", str(e))
-            child += 1
+            if isinstance(lyr_cfg, Mapping):
+                try:
+                    lyr = parse_ows_layer(lyr_cfg, global_cfg=global_cfg, parent_layer=self, sibling=child)
+                    self.unready_layers.append(lyr)
+                except ConfigException as e:
+                    _LOG.error("Could not parse layer: %s", str(e))
+                child += 1
+            else:
+                _LOG.error("Non-dictionary where dictionary expected - check for trailing comma? %s...", repr(lyr_cfg)[0:50])
         global_cfg.folder_index[obj_lbl] = self
 
     def unready_layer_count(self):
@@ -264,57 +284,6 @@ class OWSFolder(OWSLayer):
         super().make_ready(dc, *args, **kwargs)
 
 
-class CacheControlRules(OWSConfigEntry):
-    def __init__(self, cfg, layer_cfg, max_datasets):
-        super().__init__(cfg)
-        self.rules = cfg
-        self.use_caching = self.rules is not None
-        self.max_datasets = max_datasets
-        if not self.use_caching:
-            return
-        min_so_far = 0
-        max_max_age_so_far = 0
-        for rule in self.rules:
-            if "min_datasets" not in rule:
-                raise ConfigException(f"Dataset cache rule does not contain a 'min_datasets' element in layer {layer_cfg.name}")
-            if "max_age" not in rule:
-                raise ConfigException(f"Dataset cache rule does not contain a 'max_age' element in layer {layer_cfg.name}")
-            min_datasets = rule["min_datasets"]
-            max_age = rule["max_age"]
-            if not isinstance(min_datasets, int):
-                raise ConfigException(f"Dataset cache rule has non-integer 'min_datasets' element in layer {layer_cfg.name}")
-            if not isinstance(max_age, int):
-                raise ConfigException(f"Dataset cache rule has non-integer 'max_age' element in layer {layer_cfg.name}")
-            if min_datasets <= 0:
-                raise ConfigException(f"Invalid dataset cache rule in layer {layer_cfg.name}: min_datasets must be greater than zero.")
-            if min_datasets <= min_so_far:
-                raise ConfigException(f"Dataset cache rules must be sorted by ascending min_datasets values.  In layer {layer_cfg.name}.")
-            if max_datasets > 0 and min_datasets > max_datasets:
-                raise ConfigException(f"Dataset cache rule min_datasets value exceeds the max_datasets limit in layer {layer_cfg.name}.")
-            min_so_far = min_datasets
-            if max_age <= 0:
-                raise ConfigException(f"Dataset cache rule max_age value must be greater than zero in layer {layer_cfg.name}.")
-            if max_age <= max_max_age_so_far:
-                raise ConfigException(f"max_age values in dataset cache rules must increase monotonically in layer {layer_cfg.name}.")
-            max_max_age_so_far = max_age
-
-    def cache_headers(self, n_datasets):
-        if not self.use_caching:
-            return {}
-        assert n_datasets >= 0
-        if n_datasets == 0 or n_datasets > self.max_datasets:
-            return {"cache-control": "no-cache"}
-        rule = None
-        for r in self.rules:
-            if n_datasets < r["min_datasets"]:
-                break
-            rule = r
-        if rule:
-            return {"cache-control": f"max-age={rule['max_age']}"}
-        else:
-            return {"cache-control": "no-cache"}
-
-
 TIMERES_RAW = "raw"
 TIMERES_DAY = "day"
 TIMERES_MON = "month"
@@ -322,6 +291,9 @@ TIMERES_YR  = "year"
 
 TIMERES_VALS = [TIMERES_RAW, TIMERES_DAY, TIMERES_MON, TIMERES_YR]
 
+DEF_TIME_LATEST = "latest"
+DEF_TIME_EARLIEST = "earliest"
+
 
 class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
     INDEX_KEYS = ["layer"]
@@ -361,13 +333,58 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         if self.time_resolution not in TIMERES_VALS:
             raise ConfigException(
                 "Invalid time resolution value %s in named layer %s" % (self.time_resolution, self.name))
+        self.default_time_rule = cfg.get("default_time", DEF_TIME_LATEST)
+        if self.default_time_rule not in (DEF_TIME_LATEST, DEF_TIME_EARLIEST):
+            try:
+                self.default_time_rule = datetime.date.fromisoformat(self.default_time_rule)
+            except ValueError:
+                raise ConfigException(
+                    f"Invalid default_time value in named layer {self.name} ({self.default_time_rule})"
+                )
+        self.time_axis = cfg.get("time_axis")
+        if self.time_axis:
+            self.regular_time_axis = True
+            if "time_interval" not in self.time_axis:
+                raise ConfigException("No time_interval supplied in time_axis")
+            self.time_axis_interval = self.time_axis["time_interval"]
+            if not isinstance(self.time_axis_interval, int):
+                raise ConfigException("time_interval must be an integer")
+            if self.time_axis_interval <= 0:
+                raise ConfigException("time_interval must be greater than zero")
+            self.time_axis_start = self.time_axis.get("start_date")
+            self.time_axis_end = self.time_axis.get("end_date")
+            if self.time_axis_start is not None:
+                try:
+                    self.time_axis_start = datetime.date.fromisoformat(self.time_axis_start)
+                except ValueError:
+                    raise ConfigException("time_axis start_date is not a valid ISO format date string")
+            if self.time_axis_end is not None:
+                try:
+                    self.time_axis_end = datetime.date.fromisoformat(self.time_axis_end)
+                except ValueError:
+                    raise ConfigException("time_axis end_date is not a valid ISO format date string")
+            if (self.time_axis_end is not None
+                    and self.time_axis_start is not None
+                    and self.time_axis_end < self.time_axis_start):
+                raise ConfigException("time_axis end_date must be greater than or equal to the start_date if both are provided")
+        else:
+            self.regular_time_axis = False
+            self.time_axis_interval = 0
+            self.time_axis_start = None
+            self.time_axis_end = None
+
         self.dynamic = cfg.get("dynamic", False)
 
+        self.declare_unready("default_time")
         self.declare_unready("_ranges")
         self.declare_unready("bboxes")
         # TODO: sub-ranges
         self.band_idx = BandIndex(self, cfg.get("bands"))
-        self.parse_resource_limits(cfg.get("resource_limits", {}))
+        self.cfg_native_resolution = cfg.get("native_resolution")
+        self.cfg_native_crs = cfg.get("native_crs")
+        self.declare_unready("resolution_x")
+        self.declare_unready("resolution_y")
+        self.resource_limits = OWSResourceManagementRules(cfg.get("resource_limits", {}), f"Layer {self.name}")
         try:
             self.parse_flags(cfg.get("flags", {}))
             self.declare_unready("all_flag_band_names")
@@ -405,6 +422,12 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         # And finally, add to the global product index.
         self.global_cfg.product_index[self.name] = self
 
+    def time_axis_representation(self):
+        if self.regular_time_axis:
+            start, end = self.time_range(self.ranges)
+            return f"{start.isoformat()}/{end.isoformat()}/P{self.time_axis_interval}D"
+        return ""
+
     # pylint: disable=attribute-defined-outside-init
     def make_ready(self, dc, *args, **kwargs):
         self.products = []
@@ -427,6 +450,7 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         self.definition = self.product.definition
         self.force_range_update(dc)
         self.band_idx.make_ready(dc)
+        self.resource_limits.make_ready(dc)
         self.all_flag_band_names = set()
         for fb in self.flag_bands.values():
             fb.make_ready(dc)
@@ -434,6 +458,7 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
                 raise ConfigException(f"Duplicate flag band name: {fb.pq_band}")
             self.all_flag_band_names.add(fb.pq_band)
         self.ready_image_processing(dc)
+        self.ready_native_specs()
         if self.global_cfg.wcs:
             self.ready_wcs(dc)
         for style in self.styles:
@@ -446,17 +471,6 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         if not self.hide:
             super().make_ready(dc, *args, **kwargs)
 
-    # pylint: disable=attribute-defined-outside-init
-    def parse_resource_limits(self, cfg):
-        wms_cfg = cfg.get("wms", {})
-        wcs_cfg = cfg.get("wcs", {})
-        self.zoom_fill = wms_cfg.get("zoomed_out_fill_colour", [150, 180, 200, 160])
-        self.min_zoom = wms_cfg.get("min_zoom_factor", 300.0)
-        self.max_datasets_wms = wms_cfg.get("max_datasets", 0)
-        self.max_datasets_wcs = wcs_cfg.get("max_datasets", 0)
-        self.wms_cache_rules = CacheControlRules(wms_cfg.get("dataset_cache_rules"), self, self.max_datasets_wms)
-        self.wcs_cache_rules = CacheControlRules(wcs_cfg.get("dataset_cache_rules"), self, self.max_datasets_wcs)
-
     # pylint: disable=attribute-defined-outside-init
     def parse_image_processing(self, cfg):
         emf_cfg = cfg["extent_mask_func"]
@@ -532,22 +546,40 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
 
     # pylint: disable=attribute-defined-outside-init
     def parse_wcs(self, cfg):
-        if cfg is None:
+        if cfg is None or not self.global_cfg.wcs:
             self.wcs = False
             return
         else:
             self.wcs = True
+
+        if "native_resolution" in cfg:
+            if not self.cfg_native_resolution:
+                _LOG.warning(
+                    "Specifying native_resolution in wcs section of layer %s is now deprecated, please move to "
+                    "main layer section if required.", self.name)
+                self.cfg_native_resolution = cfg.get("native_resolution")
+            else:
+                _LOG.warning(
+                    "Native_resolution in wcs section of layer %s ignored in favour of value in "
+                    "main layer section.", self.name)
+
         # Native CRS
-        self.cfg_native_crs = cfg.get("native_crs")
+        if "native_crs" in cfg:
+            if not self.cfg_native_crs:
+                _LOG.warning("Specifying native_crs in wcs section of layer %s is now deprecated, pleas move to "
+                             "main layer section if required", self.name)
+                self.cfg_native_crs = cfg["native_crs"]
+            else:
+                _LOG.warning(
+                    "native_crs in wcs section of layer %s ignored in favour of value in "
+                    "main layer section.", self.name)
+
         self.declare_unready("native_CRS")
         self.declare_unready("native_CRS_def")
 
         # Rectified Grids
         self.declare_unready("origin_x")
         self.declare_unready("origin_y")
-        self.cfg_native_resolution = cfg.get("native_resolution")
-        self.declare_unready("resolution_x")
-        self.declare_unready("resolution_y")
         self.declare_unready("grid_high_x")
         self.declare_unready("grid_high_y")
         self.declare_unready("grids")
@@ -563,27 +595,68 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         else:
             self.native_format = self.global_cfg.native_wcs_format
 
+    # pylint: disable=attribute-defined-outside-init
+    def ready_native_specs(self):
+        # Native CRS
+        try:
+            self.native_CRS = self.product.definition["storage"]["crs"]
+            if self.cfg_native_crs == self.native_CRS:
+                _LOG.debug(
+                    "Native crs for layer %s is specified in ODC metadata and does not need to be specified in configuration",
+                    self.name)
+            else:
+                _LOG.warning("Native crs for layer %s is specified in config as %s - overridden to %s by ODC metadata",
+                             self.name, self.cfg_native_crs, self.native_CRS)
+        except KeyError:
+            self.native_CRS = self.cfg_native_crs
+
+        if not self.native_CRS:
+            raise ConfigException(f"No native CRS could be found for layer {self.name}")
+        if self.native_CRS not in self.global_cfg.published_CRSs:
+            raise ConfigException(
+                f"Native CRS for product {self.product_name} in layer {self.name} ({self.native_CRS}) not in published CRSs")
+        self.native_CRS_def = self.global_cfg.published_CRSs[self.native_CRS]
+
+        try:
+            # Native CRS
+            self.resolution_x = self.product.definition["storage"]["resolution"][
+                self.native_CRS_def["horizontal_coord"]]
+            self.resolution_y = self.product.definition["storage"]["resolution"][self.native_CRS_def["vertical_coord"]]
+        except KeyError:
+            self.resolution_x = None
+            self.resolution_y = None
+
+        if self.resolution_x is None:
+            try:
+                if self.cfg_native_resolution is None:
+                    raise KeyError
+                self.resolution_x, self.resolution_y = self.cfg_native_resolution
+            except KeyError:
+                raise ConfigException(
+                    f"No native resolution supplied for layer {self.name} with no product-native resolution defined in ODC."
+                )
+            except ValueError:
+                raise ConfigException(f"Invalid native resolution supplied for layer {self.name}")
+            except TypeError:
+                raise ConfigException(f"Invalid native resolution supplied for layer {self.name}")
+        elif self.cfg_native_resolution:
+            config_x, config_y = (float(r) for r in self.cfg_native_resolution)
+            if (
+                    math.isclose(config_x, float(self.resolution_x), rel_tol=1e-8)
+                    and math.isclose(config_y, float(self.resolution_y), rel_tol=1e-8)
+            ):
+                _LOG.debug(
+                    "Native resolution for layer %s is specified in ODC metadata and does not need to be specified in configuration",
+                    self.name)
+            else:
+                _LOG.warning(
+                    "Native resolution for layer %s is specified in config as %s - overridden to (%.15f, %.15f) by ODC metadata",
+                    self.name, repr(self.cfg_native_resolution), self.resolution_x, self.resolution_y)
+
     # pylint: disable=attribute-defined-outside-init
     def ready_wcs(self, dc):
         if self.global_cfg.wcs and self.wcs:
-            # Native CRS
-            try:
-                self.native_CRS = self.product.definition["storage"]["crs"]
-                if self.cfg_native_crs == self.native_CRS:
-                    _LOG.debug(
-                        "Native crs for layer %s is specified in ODC metadata and does not need to be specified in configuration",
-                        self.name)
-                else:
-                    _LOG.warning("Native crs for layer %s is specified in config as %s - overridden to %s by ODC metadata",
-                                 self.name, self.cfg_native_crs, self.native_CRS)
-            except KeyError:
-                self.native_CRS = self.cfg_native_crs
 
-            if not self.native_CRS:
-                raise ConfigException(f"No native CRS could be found for layer {self.name}")
-            if self.native_CRS not in self.global_cfg.published_CRSs:
-                raise ConfigException(f"Native CRS for product {self.product_name} in layer {self.name} ({self.native_CRS}) not in published CRSs")
-            self.native_CRS_def = self.global_cfg.published_CRSs[self.native_CRS]
             # Prepare Rectified Grids
             try:
                 native_bounding_box = self.bboxes[self.native_CRS]
@@ -597,38 +670,6 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
             self.origin_x = native_bounding_box["left"]
             self.origin_y = native_bounding_box["bottom"]
 
-            try:
-                self.resolution_x = self.product.definition["storage"]["resolution"][
-                    self.native_CRS_def["horizontal_coord"]]
-                self.resolution_y = self.product.definition["storage"]["resolution"][self.native_CRS_def["vertical_coord"]]
-            except KeyError:
-                self.resolution_x = None
-                self.resolution_y = None
-
-            if self.resolution_x is None:
-                try:
-                    if self.cfg_native_resolution is None:
-                        raise KeyError
-                    self.resolution_x, self.resolution_y = self.cfg_native_resolution
-                except KeyError:
-                    raise ConfigException(f"No native resolution supplied for WCS enabled layer {self.name}")
-                except ValueError:
-                    raise ConfigException(f"Invalid native resolution supplied for WCS enabled layer {self.name}")
-                except TypeError:
-                    raise ConfigException(f"Invalid native resolution supplied for WCS enabled layer {self.name}")
-            elif self.cfg_native_resolution:
-                config_x, config_y = (float(r) for r in self.cfg_native_resolution)
-                if (
-                        math.isclose(config_x, float(self.resolution_x), rel_tol=1e-8)
-                        and math.isclose(config_y, float(self.resolution_y), rel_tol=1e-8)
-                ):
-                    _LOG.debug(
-                        "Native resolution for layer %s is specified in ODC metadata and does not need to be specified in configuration",
-                        self.name)
-                else:
-                    _LOG.warning(
-                        "Native resolution for layer %s is specified in config as %s - overridden to (%.15f, %.15f) by ODC metadata",
-                        self.name, repr(self.cfg_native_resolution), self.resolution_x, self.resolution_y)
 
             if (native_bounding_box["right"] - native_bounding_box["left"]) < self.resolution_x:
                 ConfigException(
@@ -704,6 +745,21 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
             if self._ranges is None:
                 raise Exception("Null product range")
             self.bboxes = self.extract_bboxes()
+            if self.default_time_rule == DEF_TIME_EARLIEST:
+                self.default_time = self._ranges["start_time"]
+            elif isinstance(self.default_time_rule,
+                            datetime.date) and self.default_time_rule in self._ranges["time_set"]:
+                self.default_time = self.default_time_rule
+            elif isinstance(self.default_time_rule, datetime.date):
+                _LOG.warning("default_time for named_layer %s is explicit date (%s) that is "
+                             " not available for the layer. Using most recent available date instead.",
+                                    self.name,
+                                    self.default_time_rule.isoformat()
+                )
+                self.default_time = self._ranges["end_time"]
+            else:
+                self.default_time = self._ranges["end_time"]
+
         # pylint: disable=broad-except
         except Exception as a:
             if not self.global_cfg.called_from_update_ranges:
@@ -713,6 +769,20 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         finally:
             if not ext_dc:
                 release_cube(dc)
+
+    def time_range(self, ranges=None):
+        if ranges is None:
+            ranges = self.ranges
+        if self.regular_time_axis and self.time_axis_start:
+            start = self.time_axis_start
+        else:
+            start = ranges["times"][0]
+        if self.regular_time_axis and self.time_axis_end:
+            end = self.time_axis_end
+        else:
+            end = ranges["times"][-1]
+        return (start, end)
+
     @property
     def ranges(self):
         if self.dynamic:
@@ -781,12 +851,6 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
     def __str__(self):
         return "Named OWSLayer: %s" % self.name
 
-    def lookup(self, cfg, keyvals, subs=None):
-        if not subs and "layer" not in keyvals:
-            subs = {
-                "layer": self.product
-            }
-        return super().lookup(cfg, keyvals, subs)
     @classmethod
     def lookup_impl(cls, cfg, keyvals, subs=None):
         try:
@@ -1183,16 +1247,13 @@ class OWSConfig(OWSMetadataConfig):
             self.native_wcs_format = cfg["native_format"]
             if self.native_wcs_format not in self.wcs_formats_by_name:
                 raise ConfigException(f"Configured native WCS format ({self.native_wcs_format}) not a supported format.")
+            self.wcs_tiff_statistics = cfg.get("calculate_tiff_statistics", True)
         else:
-            self.default_geographic_CRS = None
-            self.default_geographic_CRS_def = None
             self.wcs_formats = []
             self.wcs_formats_by_name = {}
             self.wcs_formats_by_mime = {}
             self.native_wcs_format = None
-        # shouldn't need to keep these?
-        # self.dummy_wcs_grid = False
-        # self.create_wcs_grid = False
+            self.wcs_tiff_statistics = False
 
     def parse_wmts(self, cfg):
         tms_cfgs = TileMatrixSet.default_tm_sets.copy()
