@@ -120,7 +120,7 @@ def get_coverage(args):
         "Content-Type": req.format.mime,
         'content-disposition': 'attachment; filename=%s.%s' % (req.product_name, req.format.extension)
     }
-    headers.update(req.product.wcs_cache_rules.cache_headers(n_datasets))
+    headers.update(req.product.resource_limits.wcs_cache_rules.cache_headers(n_datasets))
     return (
         req.format.renderer(req.version)(req, data),
         200,
