@@ -19,6 +19,7 @@ from datacube_ows.mv_index import MVSelectOpts
 from datacube_ows.ogc_exceptions import WCS1Exception
 from datacube_ows.ogc_utils import ConfigException
 from datacube_ows.ows_configuration import get_config
+from datacube_ows.resource_limits import ResourceLimited
 
 
 class WCS1GetCoverageRequest():
@@ -109,8 +110,9 @@ class WCS1GetCoverageRequest():
         #    self.times = [parse(self.product.wcs_sole_time).date()]
         if "time" not in args:
             #      CEOS treats no supplied time argument as all time.
-            # I'm really not sure what the right thing to do is, but QGIS wants us to do SOMETHING - treat it as "now"
-            self.times = [self.product.ranges["times"][-1]]
+            # I'm really not sure what the right thing to do is, but QGIS wants us to do SOMETHING - use configured
+            # default.
+            self.times = [self.product.default_time]
         else:
             # TODO: the min/max/res format option?
             # It's a bit underspeced. I'm not sure what the "res" would look like.
@@ -162,7 +164,7 @@ class WCS1GetCoverageRequest():
                 if not b:
                     continue
                 try:
-                    self.bands.append(self.product.band_idx.band(b))
+                    self.bands.append(self.product.band_idx.locale_band(b))
                 except ConfigException:
                     raise WCS1Exception(f"Invalid measurement: {b}",
                                         WCS1Exception.INVALID_PARAMETER_VALUE,
@@ -346,10 +348,12 @@ def get_coverage_data(req):
 
             return n_datasets, data
 
-        if req.product.max_datasets_wcs > 0 and n_datasets > req.product.max_datasets_wcs:
-            raise WCS1Exception("This request processes too much data to be served in a reasonable amount of time."
-                                "Please reduce the bounds of your request and try again."
-                                "(max: %d, this request requires: %d)" % (req.product.max_datasets_wcs, n_datasets))
+        try:
+            req.product.resource_limits.check_wcs(n_datasets)
+        except ResourceLimited as e:
+            raise WCS1Exception(
+                f"This request processes too much data to be served in a reasonable amount of time. ({e}) "
+                + "Please reduce the bounds of your request and try again.")
         datasets = stacker.datasets(index=dc.index)
         output = stacker.data(datasets, skip_corrections=True)
 
@@ -404,10 +408,11 @@ def get_tiff(req, data):
             for idx, band in enumerate(data.data_vars, start=1):
                 dst.write(data[band].values, idx)
                 dst.set_band_description(idx, req.product.band_idx.band_label(band))
-                dst.update_tags(idx, STATISTICS_MINIMUM=data[band].values.min())
-                dst.update_tags(idx, STATISTICS_MAXIMUM=data[band].values.max())
-                dst.update_tags(idx, STATISTICS_MEAN=data[band].values.mean())
-                dst.update_tags(idx, STATISTICS_STDDEV=data[band].values.std())
+                if cfg.wcs_tiff_statistics:
+                    dst.update_tags(idx, STATISTICS_MINIMUM=data[band].values.min())
+                    dst.update_tags(idx, STATISTICS_MAXIMUM=data[band].values.max())
+                    dst.update_tags(idx, STATISTICS_MEAN=data[band].values.mean())
+                    dst.update_tags(idx, STATISTICS_STDDEV=data[band].values.std())
         return memfile.read()
 
 
