@@ -3,7 +3,7 @@
 #
 # Copyright (c) 2017-2021 OWS Contributors
 # SPDX-License-Identifier: Apache-2.0
-from __future__ import absolute_import, division, print_function
+import logging
 
 from flask import request
 from ows.common import WGS84BoundingBox
@@ -22,6 +22,8 @@ from datacube_ows.ows_configuration import get_config
 from datacube_ows.utils import log_call
 from datacube_ows.wcs2_utils import get_coverage_data
 
+_LOG = logging.getLogger(__name__)
+
 WCS_REQUESTS = ("DESCRIBECOVERAGE", "GETCOVERAGE")
 
 
@@ -183,20 +185,34 @@ def create_coverage_description(cfg, product):
     if product.native_CRS_def.get("vertical_coord_first"):
         axes = list(reversed(axes))
 
-    # TODO: has this setting been removed?
-    # if not product.wcs_sole_time:
-    axes.append(
-        IrregularAxis(
-            label='time',
-            index_label='k',
-            positions=[
-                f'{t.isoformat()}T00:00:00.000Z'
-                for t in product.ranges['times']
-            ],
-            uom='ISO-8601',
-            type=SpatioTemporalType.TEMPORAL,
+    if product.regular_time_axis:
+        start, end = product.time_range()
+        size = (end - start).days // product.time_axis_interval
+        axes.append(
+            RegularAxis(
+                label='time',
+                index_label='k',
+                lower_bound=start.isoformat(),
+                upper_bound=end.isoformat(),
+                resolution=f'P{product.time_axis_interval}D',
+                uom="ISO-8601",
+                type=SpatioTemporalType.TEMPORAL,
+                size=size,
+            )
+        )
+    else:
+        axes.append(
+            IrregularAxis(
+                label='time',
+                index_label='k',
+                positions=[
+                    f'{t.isoformat()}T00:00:00.000Z'
+                    for t in product.ranges['times']
+                ],
+                uom='ISO-8601',
+                type=SpatioTemporalType.TEMPORAL,
+            )
         )
-    )
 
     return CoverageDescription(
         identifier=product.name,
@@ -266,10 +282,6 @@ def desc_coverages(args):
     )
 
 
-import logging
-
-_LOG = logging.getLogger(__name__)
-
 
 @log_call
 def get_coverage(args):
