@@ -14,7 +14,7 @@ sys.path.append("/opt")
 # The location of the datcube config file.
 os.environ.setdefault("DATACUBE_CONFIG_PATH", "/opt/odc/.datacube.conf.local")
 
-from datacube_ows.ogc import app
+from datacube_ows.ogc import app  # isort:skip
 
 application = app
 
